# -*- coding: UTF-8 -*-
#  円筒形カバーの展開図作成補助スクリプト  icd_tenkaizu.py  2023.7.23  kita
# 関係スクリプト
#   icd_module.py  my_vector.py  my_module.py  zen_han.py

# 履歴
# 2023.8.1      下置文字角度値にカッコをつける
#               角度値の少数点以下桁数を設定する窓を追加
# 2023.9.20     基準線が原点を通過するかどうかの判定でgosaが大きすぎた
#               gosa= 0.00001 -> 0.0001 に変更。アルゴリズムも変更


# ICADを操るためのおまじない
import clr
clr.AddReference('sxnet')
from sxnet import *
#SxSys.init(3999)

import tkinter as tk
from tkinter import *
import tkinter.ttk as ttk

import numpy as np
from numpy import linalg as LA
import icd_module as imd
import my_vector as myv
import my_module as mmd
import os
import threading
from time import sleep
import subprocess

pi= np.pi
my_fn= os.path.basename(__file__)       # icd_*.py
save_fn= my_fn.replace('.py','.dat')    # icd_*.dat
gosa= 0.0001


def push_get(i):
    def x():
        if i== 0:   # p0 原点 R 半径
            thread = threading.Thread(target=get_p0_r, daemon=True)
            thread.start()
        elif i== 1:   # 
            pass
        elif i== 2:   # 基準線
            thread = threading.Thread(target=get_std, daemon=True)
            thread.start()
        elif i== 3:   # y0, ly 作図位置、長さ
            thread = threading.Thread(target=get_yl, daemon=True)
            thread.start()
    return x



def get_std():  # 基準線
    (v_std, p00, dvl_r, y0, ly)= get_ent()
    btn_disable()   # Button無効化
    while True:

        ss='基準線を選択してください'
        set_entry2(entry_msg, ss)
        (type_num, seg_part)= imd.get_mouse_pick2(SxSys.ENTMODE_SEG, ss)
        if type_num == 0:   # Go
            #set_entry2(entry_msg,'基準線の選択を終了しました')
            break            
        elif type_num== SxGeom.TYPE_LINE2D: # 直線 2
            if not check(seg_part, p00):    # 角度線が原点を通過するかどうかのチェック
                warning('基準線が原点を通過しません')
                break            
                
            set_entry2(entry_msg,'基準線が選択されました')
            geom = seg_part.getGeom()
            ps= np.array([geom.sp.x, geom.sp.y])    # 始点
            pe= ps+ geom.leng* np.array([geom.vec.x, geom.vec.y])    # 終点
            ls= LA.norm(ps-p00) # 原点と始点の距離
            le= LA.norm(pe-p00) # 原点と終点の距離

            #imd.p2p_line(p00, ps- p00, '橙')
            #imd.p2p_line(p00, pe- p00, '緑')

            v0= np.array([10.0, 0.0])
            if ls > le :    # 長い方を基準線にする
                th= np.rad2deg(myv.vvth_pm(v0, ps-p00))
            else:
                th= np.rad2deg(myv.vvth_pm(v0, pe-p00))
             
            set_entry2(entry[2],(str(round(th,5))))
            break 
        else:
            warning('直線以外が選択されました')
            break
    btn_normal()



def get_p0_r():  # 原点と展開半径
    btn_disable()   # Button無効化

    while True:
        ss= '円弧を選択してください。原点と半径を取得します。'
        set_entry2(entry_msg, ss)
        (type_num, seg_part)= imd.get_mouse_pick2(SxSys.ENTMODE_SEG, ss)
        #print(type_num, seg_part)   ####
        if type_num == 0:   # Go
            set_entry2(entry_msg,'原点、半径の選択を終了しました')
            break            
        elif type_num== 5 or type_num== 6: # 2次元円弧 5 円 6
            set_entry2(entry_msg,'円弧が選択されました。半径と原点を取得しました')
            geom = seg_part.getGeom()
            dvl_r= geom.radius    # 半径
            set_entry2(entry[1], str(round(dvl_r,5)))
            p00= np.array([geom.cp.x, geom.cp.y])
            set_entry2(entry[0],(str(round(p00[0],4))+', '+ str(round(p00[1],4))))
            break 
        else:
            warning('円弧以外が選択されました')
            break            
    btn_normal()

            
def get_yl():  # 作図範囲 myv.case2を使用
    (v_std, p00, dvl_r, y0, ly)= get_ent()
    btn_disable()   # Button無効化
    cnt= 1; y0= 0.0; ly= 0.0 
    while True:
        ss= '作図範囲を示す直線を選択してください No. %d' % cnt
        set_entry2(entry_msg, ss)
        (type_num, seg_part)= imd.get_mouse_pick2(SxSys.ENTMODE_SEG, ss)
        #print(type_num)
        if type_num == 0:   # Go
            set_entry2(entry_msg,'作図範囲の選択を終了しました')
            break            
        elif type_num== SxGeom.TYPE_LINE2D: # 直線 2
            if cnt== 1: # y0
                geom = seg_part.getGeom()
                ps= np.array([geom.sp.x, geom.sp.y])    # 始点
                A= ps- p00
                B= np.array([geom.vec.x, geom.vec.y])    # 範囲線１
                C= v_std
                (n, (A, B, C)) = myv.case2(A, B, C)
                
                y0= LA.norm(C[0])
                cnt+= 1
            else: # cnt== 2
                geom = seg_part.getGeom()
                ps= np.array([geom.sp.x, geom.sp.y])    # 始点
                A= ps- p00
                B= np.array([geom.vec.x, geom.vec.y])    # 範囲線２
                C= v_std
                (n, (A, B, C)) = myv.case2(A, B, C)
                ly= LA.norm(C[0])- y0
                cnt+= 1
                set_entry2(entry_msg,'作図範囲の選択を終了しました')
                break 
        else:
            warning('直線以外が選択されました')
            break            

    if y0!= 0.0 and ly!= 0.0:
        set_entry2(entry[3],(str(round(y0,4))+', '+ str(round(ly,4))))

    btn_normal()

def push_set_dim():     # 長さ寸法に角度を追加
    thread = threading.Thread(target=set_dim, daemon=True)
    thread.start()

def set_dim():    
    btn_disable()   # Button無効化
    while True:
        ss= '長さ寸法線をピックしてください'
        set_entry2(entry_msg,ss)
        (type_num, seg_part)= imd.get_mouse_pick()
        #print(type_num)
        if type_num == 0 or seg_part== None:   # Go
            set_entry2(entry_msg,'角度追加コマンドを終了しました')
            break
        elif type_num== 25: # 長さ寸法線
            add_ang(seg_part)
        else:
            warning('寸法線以外が選択されました')
            #break            
            continue   # どっちがいいか？  

    btn_normal()   # Button有効化



def add_ang(seg):   # 長さ寸法に角度を追加
    (v_std, p00, dvl_r, y0, ly)= get_ent()
    nd= int(v_ang.get())    # Number of digits
    #print(nd)

    geom = seg.getGeom()             # Segmentのデーター構造オブジェクト
    dim0 = geom.diminfo.round        # 既存の桁数 21.6.5長瀬
    opt = SxOptDimEdit()         # 寸法線編集実行オプションクラス optは継承される
    leng = geom.txtinfo.dist1
    th= np.rad2deg(leng/dvl_r)  #展開角度計算

    ss = str(round(th,nd))
    #ss = str(round(th,3))
    if ss[-1]== '0':    # ex. 15.0
        ss= ss[:-2]     # 最後の位置文字'.0'を削除  ex. 15
    down_ss= '('+ ss+ '°)'
    #print(down_ssss)                                   #外周角度
    if geom.txtinfo.downword== '':  # 下置文字が空なら
        geom.txtinfo.downword= down_ss
        #geom.txtinfo.val_refer = True                       #21.05.15 カッコをON
    else:
        geom.txtinfo.downword= ''

    geom.txtinfo.dimval_type= True         # 寸法値を実寸
    opt.round= dim0		           # 既存の桁数長瀬
    opt.dot= True                          # 小数点を半角文字にする　長瀬
    seg.editDimText(geom.txtinfo,opt)


def push_start(): #
    thread = threading.Thread(target=execute, daemon=True)
    thread.start()

def get_ent():
    th= np.deg2rad(float(entry[2].get()))   # 基準線角度[rad]
    v_std= np.array([np.cos(th), np.sin(th)])

    arr= entry[0].get().split(',')
    p00= np.array([float(arr[0]),float(arr[1])])
    dvl_r= float(entry[1].get())
    arr= entry[3].get().split(',')
    y0= float(arr[0])
    ly= float(arr[1])

    #print(v_std, p00, dvl_r, y0, ly)
    return (v_std, p00, dvl_r, y0, ly)

def sakuzu(seg):    # 原点通過を前提
    (v_std, p00, dvl_r, y0, ly)= get_ent()
    vs_= v_std/LA.norm(v_std)    # v_std単位ベクトル
    vs0= y0*vs_
    vs1= ly*vs_
    vx_= myv.rotate2(vs_,pi/2)  # xs_の法線ベクトル
     
    geom= seg.getGeom()     # Geometry情報取得
    ps= np.array([geom.sp.x, geom.sp.y])    # 始点
    pe= ps+ geom.leng* np.array([geom.vec.x, geom.vec.y])    # 終点
    ls= LA.norm(ps-p00) # 原点と始点の距離
    le= LA.norm(pe-p00) # 原点と終点の距離

    #imd.p2p_line(p00, ps- p00, '橙')
    #imd.p2p_line(p00, pe- p00, '緑')

    if ls > le :    # 長い方を対象線
        vtmp= np.array([(ps[0]- p00[0]), (ps[1]- p00[1])])
    else:
        vtmp= np.array([(pe[0]- p00[0]), (pe[1]- p00[1])])
            
    th= myv.vvth_pm(v_std, vtmp)    # 2つのVectorの内角を正負付で返す。v0 -> v1の回転角を正 
    #print(th)   ###
    if th== 0:
        p0= p00+vs0
    else:
        lx= dvl_r* th   #±あり
        vx= lx* vx_
        p0= p00+vs0+ vx

    imd.p2p_line(p0,vs1,'紫')

def check(seg, p00):    # 角度線が原点を通過するかどうかのチェック
    geom= seg.getGeom()     # Geometry情報取得
    p1= np.array([geom.sp.x, geom.sp.y])
    p2= p1+ geom.leng* np.array([geom.vec.x, geom.vec.y])
    #if all(p1== p00)== True or all(p2== p00)== True:  # p00と重なる
    if LA.norm(p1- p00) < gosa or LA.norm(p2- p00) < gosa :  # p00と重なる
        #print('重なる')
        return True

    v1= p1- p00
    v2= p2- p00
    #tmp = np.arccos.(np.dot(v1,v2)/LA.norm(p1)/ LA.norm(p2))     # 内角
    tmp= myv.vvth(v1, v2)   # 内角
    l0x= LA.norm(p1)* np.sin(tmp)
    #print(tmp, l0x)
    if l0x < gosa:
        return True
    else:
        print('\n   原点と距離 %.5f 離れています\n' % l0x)
        return False

def check_bak(seg, p00):    # 角度線が原点を通過するかどうかのチェック
    geom= seg.getGeom()     # Geometry情報取得
    p1= np.array([geom.sp.x, geom.sp.y, 0.0])
    p2= np.array([geom.vec.x, geom.vec.y])
    pe= p1+ geom.leng* p2
    if LA.norm(p1- p00) < gosa or LA.norm(pe- p00) < gosa :  # p00と重なる
        print('重なる')
        return True
    v0= myv.rotate2(p2, pi/2)
    v0= np.array([v0[0], v0[1], 0.0])
    p00= np.array([p00[0], p00[1], 0.0])

    l0x= myv.f2plen(v0, p1, p00) 
    print(l0x, gosa)
    if l0x < gosa:
        return True
    else:
        print('\n   距離 %.6f 離れています\n' % l0x)
        return False

def warning(ss):
    set_entry2(entry_msg,ss)
    print('\007')
    entry_msg.configure(bg= 'orange')
    sleep(2)
    entry_msg.configure(bg= e_bcol)

def execute():  # 角度線選択 
    i= 1
    (v_std, p00, dvl_r, y0, ly)= get_ent()
    btn_disable()   # Button無効化
    while True:
        ss= '角度線 N0.% d をピックしてください' % i
        set_entry2(entry_msg,ss)
        (type_num, seg_part)= imd.get_mouse_pick2(SxSys.ENTMODE_SEG, ss)
        
        #print(type_num,seg_part)    ####
        
        if type_num == 0 or seg_part== None:   # Go or Comand選択
            set_entry2(entry_msg,'角度線の選択を終了しました')
            break
        elif type_num== SxGeom.TYPE_LINE2D: # 直線 2
            if check(seg_part, p00):    # 角度線が原点を通過するかどうかのチェック
                sakuzu(seg_part)
                i+= 1
            else:
                warning('角度線が原点を通過しません')

        else:
            warning('角度線以外が選択されました')
            continue
            # break
    btn_normal()


def btn_disable():
    for i in range(len(btn)):
        btn[i]['state'] = 'disabled'


def btn_normal():
    for i in range(len(btn)):
        btn[i]['state'] = 'normal'
        #btn['state'] = 'normal'



def set_data():
    fp = open(save_fn, 'r')
    lines = fp.readlines()
    fp.close()
    i= 0
    for line in lines:
        line = line.rstrip()    # 行末の改行削除
        if line.find('#') > -1:  # #が含まれてるなら
            continue

        tmp = line.split('= ')
        #print(line) #!!!!
        if line.find('entry') > -1: # Entry行
            #if len(tmp) == 2: 
            set_entry(i, tmp[1])
            i+= 1
    

def set_entry(idx, ss): # Entryに文字列を挿入
    entry[idx].delete(0,tk.END)
    entry[idx].insert(0,ss)

def set_entry2(ent, ss): # Entryに文字列を挿入
    ent.delete(0,tk.END)
    ent.insert(0,ss)


def save_data():    # 設定値保存
    ss = ''
    for i in range(len(entry)):
        #print(i)
        ss = ss + 'entry[%d]= %s\n' % (i, entry[i].get())

    fp = open(save_fn,'w')
    fp.write(ss)
    fp.close()    

def on_closing():       # 終了時の処理
    save_data()
    ss_geom = root.geometry() 
    mmd.save_geometry(save_fn_geom, ss_geom)

    #print('\n   %s を終了しました\n' % tmp_fn)
    print('\n   %s を終了しました\n' % my_fn)

    root.destroy()


#--------------------------------
# ヘルプボタン
#--------------------------------
def push_help():
    cmd = 'start Acrobat icd_tenkaizu.pdf'
    result = subprocess.call(cmd,shell=True)


#************************************
# ボタン等の部品配置
#************************************
nl = 7
root = tk.Tk()
root.attributes("-topmost", True)    # 常に前面に表示
root.configure(bg='RoyalBlue4')
root.title(my_fn)  #タイトル
save_fn_geom = my_fn.replace('.py', '.ini')
ss_geom = mmd.read_geometry(save_fn_geom)
if ss_geom == None:
    ss_geom = '400x%d+1000+0' % (nl*26)

root.geometry(ss_geom)

btn_bcol = 'grey21'     #ボタン背景色
btn_tcol = 'white'      #ボタン文字色
label_bcol = 'khaki3'   #ラベル背景色
label_tcol = 'black'    #ラベル文字色
e_bcol = 'ivory2'       #Entry背景色

x0= 10; y0= 2
dx= 100; dy= 22
px= 2; py= 2
ie=0

frame1 = tk.LabelFrame(root, text='作図パラメータ', padx=px, pady=py,bg="RoyalBlue3",fg=btn_tcol) 
frame1.place(x=x0,y=y0)
label_txt= ['1. 原点 ＆','展開半径','2.基準線角度','3.作図範囲線(y0,ly)']
btn_txt= ['GetP0_R','     ','Get Std','Get Y'] 
nn = len(label_txt)
entry=[]; btn= []
for i in range(nn):
    label= tk.Label(frame1, text=label_txt[i], bg=label_bcol,fg=label_tcol)
    label.grid(column=0,row=i, padx=px, pady=py)
    entry.append(tk.Entry(frame1, width=12,justify="right",bg=e_bcol))
    if i== 0 or i== 3:
        entry[ie].insert(0,str('%d, %d' % (i, i)))
        entry[ie].grid(column=1,row=i, padx=px, pady=py); ie += 1
    else:
        entry[ie].insert(0,str(ie)) 
        entry[ie].grid(column=1,row=i, padx=px, pady=py); ie += 1


    if i != 1:
        btn.append(tk.Button(frame1,text = btn_txt[i],command = push_get(i),bg=btn_bcol,fg=btn_tcol))
        btn[-1].grid(column=2,row=i, padx=px, pady=py)

# メッセージ用 Entry
entry_msg=(tk.Entry(root, width=50,justify="right",bg=e_bcol)) 
entry_msg.place(x= x0, y= dy*nl)



#---------------------------------------------------
ss = 'ヘルプ'   #実行Buttn
#---------------------------------------------------
btn.append(tk.Button(root,text= ss,command= push_help,bg=btn_bcol,fg=btn_tcol))
btn[-1].place(x=350,y=y0)

#---------------------------------------------------
ss = '角度桁数'
#---------------------------------------------------
label= tk.Label(root, text=ss, bg=label_bcol,fg=label_tcol)
label.place(x=280,y=55)

# Combobox
v_ang= StringVar()
cb_ang = ttk.Combobox(root, width=2, textvariable=v_ang)
cb_ang['values']=('0','1','2','3')
cb_ang.current(2)
cb_ang.place(x=340, y=55)

#---------------------------------------------------
ss = '長さ寸法に角度追加'    #実行Buttn
#---------------------------------------------------
btn.append(tk.Button(root,text= ss,command= push_set_dim,bg='orange',fg='black')) #ボタンの作成
btn[-1].place(x=280,y=80)

#---------------------------------------------------
ss = '展開ライン作図';    #実行Buttn
#---------------------------------------------------
btn.append(tk.Button(root,text= ss,command= push_start,bg=btn_bcol,fg=btn_tcol)) #ボタンの作成
btn[-1].place(x=280,y=110)

#--------------------------------
#   起動時の処理
#--------------------------------
# icd_tenkaizu.datがあれば読み込んでパラメータセット
if os.path.exists(save_fn) == True:
    print('\n %s を読み込みます' % save_fn)
    set_data()



root.protocol("WM_DELETE_WINDOW", on_closing)
root.mainloop()
